@@ -36,15 +36,15 @@ use strict;
 
 package FAQ::OMatic::Appearance;
 use FAQ::OMatic::ImageRef;
+use FAQ::OMatic::I18N;
 
-use vars qw($editStart $editEnd $otherStart $otherEnd $highlightColor
-	$highlightStart $highlightEnd $graphHistory $graphHeight $graphWidth);
+use vars qw($highlightColor $highlightStart $highlightEnd
+	$graphHistory $graphHeight $graphWidth);
+	# basically constants. TODO mod_perl -- they're configs, so when admin
+	# changes config, it won't show up immediately in mod_perl children.
 
-# These surround the editing buttons. I make them smaller so they'll
-# not look as much like part of the item being displayed, and more
-# like little intruders.
-$editStart = "<font size=-1>";
-$editEnd = "</font>";
+my @allLinks;		# constants. no mod_perl cache badness
+my $indentTypes;
 
 # These surround words in the document that were in a search query.
 $highlightColor	= $FAQ::OMatic::Config::highlightColor || "#a01010";
@@ -60,54 +60,79 @@ $graphHeight	= 180;
 # the homepage of Faq-O-Matic so others can see where to get the
 # software for their own site.
 sub cPageHeader {
-	my $params = shift;
+	my $params = shift || {};
+	my $showLinks = shift || [];
 	my $suppressType = shift || '';
+
 	# this is a func because FAQ::OMatic::fomTitle() isn't well-defined at
 	# global initialization time.
 	my $type = ($suppressType) ? '' : "Content-type: text/html\n\n";
 
 	# THANKS: to Billy Naylor for requesting the ability to insert
 	# THANKS: a corporate logo into every page's HTML.
-	my $pageHeader = $FAQ::OMatic::Config::pageHeader || '';
-	return $type
-			."<html><head><title>".FAQ::OMatic::fomTitle()
+	if (FAQ::OMatic::getParam($params, 'render') ne 'text') {
+		my $pageHeader = $FAQ::OMatic::Config::pageHeader || '';
+		my $page = '';
+		$page .= $type;
+		$page .= "<html><head><title>".FAQ::OMatic::fomTitle()
 			.FAQ::OMatic::pageDesc($params)."</title></head>\n"
 			."<body bgcolor=\"$FAQ::OMatic::Config::backgroundColor\" "
 			."text=$FAQ::OMatic::Config::textColor "
 			."link=$FAQ::OMatic::Config::linkColor "
 			."vlink=$FAQ::OMatic::Config::vlinkColor>\n$pageHeader\n";
+		if ($FAQ::OMatic::Config::navigationBlockAtTop || '') {
+			# THANKS to Jim Adler <jima@sr.hp.com> for suggesting
+			# a copy of the nav block at the top of each page.
+			$page .= navigationBlock($params, $showLinks);
+		}
+		return $page;
+	} else {
+		my $title = FAQ::OMatic::fomTitle().FAQ::OMatic::pageDesc($params);
+		my $space = " "x(int((75-length($title))/2));
+		return $space.$title."\n\n";
+	}
 }
 
-# TODO: reorganize cPageFooter to be able to render simple HTML
-# (no tables, just a <hr> in front of the links) and even text
-#	my $render = $params->{'render'} || 'fancy';
-
 sub cPageFooter {
-	my $params = shift;		# hash ref
+	my $params = shift || {};
+	my $showLinks = shift || [];
+
+	if (FAQ::OMatic::getParam($params, 'render') eq 'text') {
+		return "Generated by FAQ-O-Matic $FAQ::OMatic::VERSION,\n"
+			."available at "
+			."http://www.dartmouth.edu/cgi-bin/cgiwrap/jonh/faq.pl\n"
+	} else {
+		my $page = '';
+		$page .= navigationBlock($params, $showLinks);
+		$page .= $FAQ::OMatic::Config::pageFooter || '';
+		$page .= "</body></html>\n";
+		return $page;
+	}
+}
+
+@allLinks =
+	( 'help', 'search', 'appearance', 'entire', 'edit', 'renderText' );
+
+sub allLinks {
+	my @a2 = @allLinks;	 # make a copy of the array so it doesn't get modified
+	return \@a2;
+}
+
+sub navigationBlock {
+	my $params = shift;
 	my $showLinks = shift || [];	# ref to array of links to show
 
 	my $filename = $params->{'file'} || '1';
 	my $recurse = $params->{'_recurse'} || '';
 	my $item = new FAQ::OMatic::Item($filename);
 
-	my $boxColor = '';	# TODO: I don't think I use this anymore, anyway.
-
-	my @sl;
-	if ($showLinks eq 'all') {
-		@sl = ('help', 'search', 'appearance', 'entire', 'edit');
-	} elsif (ref $showLinks) {
-		@sl = @{$showLinks};
-	}
-	my %sl = map {$_=>$_} @sl;
+	my %sl = map {$_=>$_} @{$showLinks};
 	$showLinks = \%sl;
 
-	my $page = "\n<!--footer box at bottom of page with common links -->\n";
-	$page .="<table width=\"100%\" "
-		."bgcolor=\"$FAQ::OMatic::Config::regularPartColor\">\n"
-		."<tr><td><table width=\"100%\">\n";
+	delete $showLinks->{'renderText'}
+		if (FAQ::OMatic::getParam($params, 'textCmds') eq 'hide');
 
 	my @cells = ();
-
 	if ($showLinks->{'help'}) {
 		push @cells, helpButton($params);
 	}
@@ -115,23 +140,19 @@ sub cPageFooter {
 	if ($showLinks->{'search'}) {
 		# Search Form
 		push @cells,
-			"<td align=center $boxColor>"
-			.FAQ::OMatic::button(
+			FAQ::OMatic::button(
 				FAQ::OMatic::makeAref('-command'=>'searchForm',
 					'-params'=>$params),
-				"Search</a>")
-			."</td>\n";
+				gettext("Search") . "</a>");
 	}
 
 	if ($showLinks->{'appearance'}) {
 		# Appearance Options
 		push @cells,
-			"<td align=center $boxColor>"
-			.FAQ::OMatic::button(
+			FAQ::OMatic::button(
 				FAQ::OMatic::makeAref('-command'=>'appearanceForm',
 					'-params'=>$params),
-			"Appearance")
-			."</td>\n";
+			gettext("Appearance"));
 	}
 
 	if ($showLinks->{'entire'}) {
@@ -141,47 +162,61 @@ sub cPageFooter {
 				# provide a way to get rid of the recursive display
 				# THANKS: Jim Adler <jima@sr.hp.com>
 				push @cells,
-					"<td align=center $boxColor>"
-					.FAQ::OMatic::button(
+					FAQ::OMatic::button(
 						FAQ::OMatic::makeAref('-command'=>'faq',
 							'-params'=>$params,
 							'-changedParams'=>{'_recurse'=>''}),
-						"Show Top Category Only</a>")
-					."</td>\n";
+						gettext("Show Top Category Only") . "</a>");
 			} else {
 				push @cells,
-				"<td align=center $boxColor>"
-					.FAQ::OMatic::button(
+					FAQ::OMatic::button(
 						FAQ::OMatic::makeAref('-command'=>'faq',
 							'-params'=>$params,
 							'-changedParams'=>{'_recurse'=>1}),
-						"Show This <em>Entire</em> Category</a>")
-					."</td>\n";
+						gettext("Show This <em>Entire</em> Category") . "</a>");
 			}
 		} else {
-			push @cells, "<td align=center></td>\n";
+			push @cells, "";
+		}
+	}
+
+	if ($showLinks->{'renderText'}) {
+		my $whatAmI = gettext($item->whatAmI());
+		push @cells,
+			FAQ::OMatic::button(
+				FAQ::OMatic::makeAref('-command'=>'faq',
+					'-params'=>$params,
+					'-changedParams'=>{'render'=>'text'}),
+				gettext("Show This $whatAmI As Text</a>"));
+		if ($item->isCategory() and $showLinks->{'entire'}) {
+			push @cells,
+				FAQ::OMatic::button(
+					FAQ::OMatic::makeAref('-command'=>'faq',
+						'-params'=>$params,
+						'-changedParams'=>{'render'=>'text', '_recurse'=>1}),
+				 gettext("Show This <em>Entire</em> Category As Text</a>"));
 		}
 	}
 
 	if ($showLinks->{'edit'}
 		and $FAQ::OMatic::Config::showEditOnFaq) {
 		# Show Edit Commands
-		my $editcell = "<td align=center $boxColor>";
-		if ($params->{'showEditCmds'}) {
-			$editcell.=FAQ::OMatic::button(
+		if (FAQ::OMatic::getParam($params, 'editCmds') ne 'hide') {
+			push @cells, FAQ::OMatic::button(
 				FAQ::OMatic::makeAref('-command'=>'faq',
 					'-params'=>$params,
-					'-changedParams'=>{'showEditCmds'=>''}),
-				"Hide Expert Edit Commands");
+					'-changedParams'=>{'editCmds'=>'hide'}),
+				gettext("Hide Expert Edit Commands"));
 		} else {
-			$editcell.=FAQ::OMatic::button(
+			my $showStyle = $FAQ::OMatic::Config::showEditOnFaq || 'show';
+			$showStyle = 'show' if ($showStyle ne 'compact');
+
+			push @cells, FAQ::OMatic::button(
 				FAQ::OMatic::makeAref('-command'=>'faq',
 					'-params'=>$params,
-					'-changedParams'=>{'showEditCmds'=>'1'}),
-				"Show Expert Edit Commands");
+					'-changedParams'=>{'editCmds'=>$showStyle}),
+				gettext("Show Expert Edit Commands"));
 		}
-		$editcell.="</td>\n";
-		push @cells, $editcell;
 	}
 
 	if ($showLinks->{'faq'}) {
@@ -189,32 +224,47 @@ sub cPageFooter {
 		my $cmd = $params->{'cmd'} || '';
 		if ($cmd ne '' and $cmd ne 'faq') {
 			push @cells,
-				"<td align=center $boxColor>"
-				.FAQ::OMatic::button(
+				FAQ::OMatic::button(
 					FAQ::OMatic::makeAref('-command'=>'faq',
 						'-params'=>$params,
 						# kill unneeded params from 'authenticate':
 						'-changedParams'=>{'partnum'=>'',
 							'checkSequenceNumber'=>''},
 					),
-					"Return to FAQ");
+					gettext("Return to FAQ"));
 		}
 	}
 
-	$page .= "<tr>\n".join('', @cells)."</tr>\n";
+	my $useTable = FAQ::OMatic::getParam($params, 'render') eq 'tables';
 
-	my $numCells = scalar(@cells) || 0;
-	$page.=	"<tr><td colspan=$numCells align=center>\n"
-			."This is <a href=\""
+	my $page = "\n<!--footer box at bottom of page with common links -->\n";
+	my $software = gettext("This is a") . " <a href=\""
 			."http://www.dartmouth.edu/cgi-bin/cgiwrap/jonh/faq.pl"
-			."\">Faq-O-Matic</a> $FAQ::OMatic::VERSION.\n"
-			."</td></tr></table>\n"
+			."\">Faq-O-Matic</a> $FAQ::OMatic::VERSION.\n";
+	if ($useTable) {
+		my $tw = $FAQ::OMatic::Config::tableWidth || '';
+		my $bgc = $FAQ::OMatic::Config::regularPartColor || '#ffffff';
+		$page .="<table $tw "
+			."bgcolor=\"$bgc\">\n"
+			."<tr><td><table $tw>\n";
+		@cells = map { "<td>$_</td>\n" } @cells;
+		$page .= "<tr>\n".join('', @cells)."</tr>\n";
+		my $numCells = scalar(@cells) || 0;
+		if ($showLinks->{'faqomatic-home'}) {
+			$page.=	"<tr><td colspan=$numCells align=center>\n"
+				.$software
+				."</td></tr>";
+		}
+		$page .= "</table>\n"
 			."</td></tr></table>\n";
+	} else {
+		@cells = map { "<br>$_\n" } @cells;
+		$page .= "\n".join('', @cells)."\n";
+		if ($showLinks->{'faqomatic-home'}) {
+			$page .= "<br>".$software;
+		}
+	}
 
-	$page .= $FAQ::OMatic::Config::pageFooter || '';
-
-	$page .= "</body></html>\n";
-	
 	return $page;
 }
 
@@ -278,6 +328,8 @@ sub itemRender {
 	# $itemboxes->[i]->{'rows'}->[p] is a ref to a hash, describing that part.
 	# $itemboxes->[i]->{'rows'}->[p]->{'type'} is one of
 	#	'three', 'multirow', 'wide'.
+	# $itemboxes->[i]->{'rows'}->[p]->{'id'} is a debugging string that
+	#	indicates the source of the row data
 	# type 'three' parts have ->{'body'}, ->{'editbody'}, ->{'afterbody'}
 	#	refs. 'body' is a hash ref to 'text' and 'color'.
 	#	'editbody' is an array ref to edit cmds that apply to this part body.
@@ -289,11 +341,14 @@ sub itemRender {
 	# type 'wide' fields have ->{'text'} and ->{'color'} parts that should
 	#	fill the width of the display.
 
-	my $render = $params->{'render'} || 'fancy';
+	my $render = FAQ::OMatic::getParam($params, 'render');
 	if ($render eq 'simple') {
 		return itemRenderSimple($params, $itemboxes);
+	} elsif ($render eq 'text') {
+		return itemRenderText($params, $itemboxes);
 	} else {
-		my $editDisplay = $params->{'showEditCmds'} || '';
+		# tables
+		my $editDisplay = FAQ::OMatic::getParam($params, 'editCmds');
 		if ($editDisplay eq 'compact') {
 			return itemRenderCompactEdits($params, $itemboxes);
 		} else {
@@ -337,7 +392,8 @@ sub itemRenderNormalEdits {
 
 	my $rt = '';
 
-	$rt.= "<table width=100% cellpadding=5 cellspacing=2>\n";
+	$rt.= "<table $FAQ::OMatic::Config::tableWidth "
+		."cellpadding=5 cellspacing=2>\n";
 	foreach my $itembox (@{$itemboxes}) {
 		my $item = $itembox->{'item'};
 		my $rows = $itembox->{'rows'};
@@ -423,9 +479,9 @@ sub getColorText {
 	$color = "bgcolor=$color" if ($color);
 	my $text = $hashref->{'text'} || '';
 	if ($size eq 'edit') {
-		$text = $FAQ::OMatic::Appearance::editStart
-			.$text
-			.$FAQ::OMatic::Appearance::editEnd;
+		# The editing buttons are smaller so that they'll not look as much like
+		# part of the item being displayed, but more like little intruders.
+		$text = "<font size=-1>${text}</font>";
 	}
 	return ($color,$text);
 }
@@ -469,7 +525,8 @@ sub itemRenderCompactEdits {
 
 	my $rt = '';
 
-	$rt.= "<table width=100% cellpadding=5 cellspacing=2>\n";
+	$rt.= "<table $FAQ::OMatic::Config::tableWidth "
+		."cellpadding=5 cellspacing=2>\n";
 	foreach my $itembox (@{$itemboxes}) {
 		my $item = $itembox->{'item'};
 		my $rows = $itembox->{'rows'};
@@ -480,6 +537,9 @@ sub itemRenderCompactEdits {
 	
 		my $itemFile = $item->{'filename'};
 		my $itemName = $item->getTitle();
+		# THANKS to charlie buckheit <buckheit@olg.com> for suggesting
+		# the width tag, which helps keep the item-tall bar skinny
+		# in Internet Exploder. (Nothing seems to help in Netscape.)
 		$rt.="\n<!--Item: $itemName file: $itemFile--><tr>\n"
 				."<td bgcolor=$FAQ::OMatic::Config::itemBarColor "
 				."valign=top align=center rowspan=$tablerows width=$sw>\n"
@@ -533,7 +593,7 @@ sub itemRenderCompactEdits {
 					my ($color,$text) = getColorText($cell);
 					$rt .= "<!--cell-->".$text;
 				}
-				$rt .= "</tr>\n";
+				$rt .= "</td></tr>\n";
 			} else {
 				# row is specified as a single cell that should fill the
 				# width of the table.
@@ -618,6 +678,85 @@ sub itemRenderSimple {
 	return $rt;
 }
 
-# TODO: eventually, an itemRenderText function.
+sub itemRenderText {
+	# a text-only rendering mode
+	my $params = shift;
+	my $itemboxes = shift;
+
+	my $rt = '';
+
+	foreach my $itembox (@{$itemboxes}) {
+		my $item = $itembox->{'item'};
+		my $rows = $itembox->{'rows'};
+
+		my $itemFile = $item->{'filename'};
+		my $itemName = $item->getTitle();
+
+		# this rendering method assumes (hopes!) that the first
+		# row of an item is a 'wide' row, something that looks like
+		# a title.
+		my $row = shift @$rows;
+		if ($row->{'type'} ne 'wide') {
+			FAQ::OMatic::gripe('problem', "assertion failed. ".caller(0));
+		}
+		my $text = $row->{'text'};
+		$rt.="$text\n";
+
+		foreach $row (@{$rows}) {
+			my $part = $row->{'part'};
+			my $indentType = 'regular'; 
+			if (defined $part and ($part->{'Type'} eq 'directory')) {
+				$indentType = 'directory';
+			}
+				
+			if ($row->{'type'} eq 'three') {
+				my ($bodycolor,$bodytext) = getColorText($row->{'body'});
+				my @editbody = @{$row->{'editbody'}};	# array ref
+				my @afterbody = @{$row->{'afterbody'}};	# array ref
+
+				$rt.=indent($indentType, $bodytext);
+				# edit text not shown (supported) in render=text mode
+				# (that's the editbody and afterbody data)
+				$rt .= "\n";
+			} elsif ($row->{'type'} eq 'multirow') {
+				# row is specified as a series of cells to be crammed
+				# together horizontally.
+				if (not $row->{'isEdit'}) {
+					# supress data that's just editing links
+					foreach my $cell (@{$row->{'cells'}}) {
+						my ($color,$text) = getColorText($cell);
+						$rt .= indent($indentType,$text);
+					}
+				}
+			} else {
+				# row is specified as a single cell that should fill the
+				# width of the table.
+				my ($color,$text) = getColorText($row);
+				$rt .= indent($indentType,$text);
+			}
+		}
+		$rt.="\n";
+	}
+	$rt.="\n";
+
+	return $rt;
+}
+
+$indentTypes = {
+	'regular'	=> '    ',
+	'directory'	=> '  + ',
+};
+
+sub indent {
+	my $type = shift;
+	my $text = shift;
+
+	my $indent = defined($indentTypes->{$type})
+		? $indentTypes->{$type}
+		: $type;
+
+	$text =~ s#^#$indent#gm;
+	return $text;
+}
 
 1;
